@@ -14,6 +14,7 @@
 #' @inheritParams GetSysMat
 #' @inheritParams StateSpaceEval
 #' @inheritParams Cholesky
+#' @inheritParams Cycle
 #'
 #' @return
 #' A list containing the system matrices.
@@ -24,7 +25,8 @@ ARIMA <- function(p = 1,
                   exclude_arima = NULL,
                   fixed_part = TRUE,
                   update_part = TRUE,
-                  param = rep(1, p^2 + 0.5 * p * (p+1)),
+                  transform_only = FALSE,
+                  param = rep(1, p^2 + 0.5 * p * (p + 1)),
                   decompositions = TRUE,
                   T1 = NULL,
                   T2 = NULL,
@@ -34,14 +36,18 @@ ARIMA <- function(p = 1,
 
   # Check for erroneous input
   if (length(arima_spec) != 3) {
-    stop("The ARIMA specification must be a vector of length 3.")
+    stop(
+      "The ARIMA specification must be a vector of length 3.",
+      call. = FALSE
+    )
   }
-  if (arima_spec[2] < 0) {
+  if (arima_spec[[2]] < 0) {
     stop(
       paste(
         "Number of differencing of the ARIMA component must",
         "be greater than or equal to 0."
-      )
+      ),
+      call. = FALSE
     )
   }
 
@@ -52,7 +58,7 @@ ARIMA <- function(p = 1,
   result <- list()
 
   # Number of stationary elements in the state vector per included dependent
-  r <- max(arima_spec[1], arima_spec[3] + 1)
+  r <- max(arima_spec[[1]], arima_spec[[3]] + 1)
 
   if (fixed_part) {
 
@@ -61,35 +67,38 @@ ARIMA <- function(p = 1,
     if (n_arima < p) {
       Z <- Z[, -exclude_arima, drop = FALSE]
     }
-    if (arima_spec[2] > 0) {
-      Z <- do.call(cbind, replicate(1 + arima_spec[2], Z, simplify = FALSE))
+    if (arima_spec[[2]] > 0) {
+      Z <- do.call(cbind, replicate(1 + arima_spec[[2]], Z, simplify = FALSE))
     }
     Z <- cbind(Z, matrix(0, p, n_arima * (r - 1)))
     result$Z <- Z
 
     # Initialisation of the State vector and corresponding uncertainty
-    result$a1 <- matrix(0, (r + arima_spec[2]) * n_arima, 1)
+    result$a1 <- matrix(0, (r + arima_spec[[2]]) * n_arima, 1)
     result$P_inf <- BlockMatrix(
-      diag(1, arima_spec[2] * n_arima, arima_spec[2] * n_arima),
+      diag(1, arima_spec[[2]] * n_arima, arima_spec[[2]] * n_arima),
       matrix(0, r * n_arima, r * n_arima)
     )
 
     # T and R matrices are fixed if no coefficients are needed
-    if (arima_spec[1] == 0 & arima_spec[3] == 0) {
+    if (arima_spec[[1]] == 0 && arima_spec[[3]] == 0) {
       T1 <- matrix(0, n_arima, n_arima)
-      if (arima_spec[2] > 0) {
+      if (arima_spec[[2]] > 0) {
         T1 <- cbind(
-          matrix(0, n_arima, arima_spec[2] * n_arima),
+          matrix(0, n_arima, arima_spec[[2]] * n_arima),
           T1
         )
         T2 <- diag(1, n_arima, n_arima)
-        T3 <- matrix(0, 0, n_arima + arima_spec[2] * n_arima)
-        for (i in arima_spec[2]:1) {
+        T3 <- matrix(0, 0, n_arima + arima_spec[[2]] * n_arima)
+        for (i in arima_spec[[2]]:1) {
           T3 <- rbind(
             T3,
             cbind(
-              matrix(0, n_arima, (arima_spec[2] - i) * n_arima),
-              do.call(cbind, replicate(i, diag(1, n_arima, n_arima), simplify = FALSE)),
+              matrix(0, n_arima, (arima_spec[[2]] - i) * n_arima),
+              do.call(
+                cbind,
+                replicate(i, diag(1, n_arima, n_arima), simplify = FALSE)
+              ),
               T2
             )
           )
@@ -98,7 +107,7 @@ ARIMA <- function(p = 1,
       }
       result$Tmat <- T1
       result$R <- rbind(
-        matrix(0, arima_spec[2] * n_arima, n_arima),
+        matrix(0, arima_spec[[2]] * n_arima, n_arima),
         diag(1, n_arima, n_arima)
       )
     } else {
@@ -114,26 +123,29 @@ ARIMA <- function(p = 1,
       result$R1 <- R1
 
       # Non-stationary part of Tmat and R
-      if (arima_spec[2] > 0) {
-        T2 <- matrix(0, dim(T1)[1], arima_spec[2] * n_arima)
+      if (arima_spec[[2]] > 0) {
+        T2 <- matrix(0, dim(T1)[[1]], arima_spec[[2]] * n_arima)
         Ttemp <- cbind(
           diag(1, n_arima, n_arima),
-          matrix(0, n_arima, dim(T1)[2] - n_arima)
+          matrix(0, n_arima, dim(T1)[[2]] - n_arima)
         )
-        T3 <- matrix(0, 0, dim(T1)[2] + dim(T2)[2])
-        for (i in arima_spec[2]:1) {
+        T3 <- matrix(0, 0, dim(T1)[[2]] + dim(T2)[[2]])
+        for (i in arima_spec[[2]]:1) {
           T3 <- rbind(
             T3,
             cbind(
-              matrix(0, n_arima, (arima_spec[2] - i) * n_arima),
-              do.call(cbind, replicate(i, diag(1, n_arima, n_arima), simplify = FALSE)),
+              matrix(0, n_arima, (arima_spec[[2]] - i) * n_arima),
+              do.call(
+                cbind,
+                replicate(i, diag(1, n_arima, n_arima), simplify = FALSE)
+              ),
               Ttemp
             )
           )
         }
         result$T2 <- T2
         result$T3 <- T3
-        R2 <- matrix(0, arima_spec[2] * n_arima, n_arima)
+        R2 <- matrix(0, arima_spec[[2]] * n_arima, n_arima)
         result$R2 <- R2
       }
     }
@@ -144,12 +156,12 @@ ARIMA <- function(p = 1,
     # Check for number of parameters
     param <- param[!is.na(param)]
     needed <- 0.5 * n_arima * (n_arima + 1) +
-      (arima_spec[1] + arima_spec[3]) * n_arima^2
+      (arima_spec[[1]] + arima_spec[[3]]) * n_arima^2
     if (length(param) < needed) {
-      stop("Not enough parameters supplied.")
+      stop("Not enough parameters supplied.", call. = FALSE)
     }
     if (length(param) > needed) {
-      stop("Too many parameters supplied.")
+      stop("Too many parameters supplied.", call. = FALSE)
     }
 
     # Parameters for the variance - covariance matrix
@@ -175,7 +187,7 @@ ARIMA <- function(p = 1,
     }
 
     # T matrix and coefficients in R matrix
-    if (arima_spec[1] > 0 | arima_spec[3] > 0) {
+    if (arima_spec[[1]] > 0 || arima_spec[[3]] > 0) {
 
       # Coefficients
       if (n_arima == 1) {
@@ -183,78 +195,93 @@ ARIMA <- function(p = 1,
       } else {
         A <- array(
           param[-Q_indices],
-          dim = c(n_arima, n_arima, arima_spec[1] + arima_spec[3])
+          dim = c(n_arima, n_arima, arima_spec[[1]] + arima_spec[[3]])
         )
       }
       coeff <- CoeffARMA(
         A = A, variance = result$Q,
-        ar = arima_spec[1], ma = arima_spec[3]
+        ar = arima_spec[[1]], ma = arima_spec[[3]]
       )
 
       # T matrix coefficients
-      if (arima_spec[1] > 0) {
-        result$ar <- coeff$ar
-        if (n_arima > 1) {
-          coeff$ar <- aperm(coeff$ar, c(2, 1, 3))
+      if (arima_spec[[1]] > 0) {
+        if (decompositions) {
+          result$ar <- coeff$ar
         }
-        T1[1:(n_arima * arima_spec[1]), 1:n_arima] <- t(
-          matrix(
-            coeff$ar,
-            n_arima,
-            n_arima * arima_spec[1]
-          )
-        )
-      }
-      T_stationary <- T1
-
-      # R matrix coefficients
-      if (arima_spec[3] > 0) {
-        result$ma <- coeff$ma
-        if (n_arima > 1) {
-          coeff$ma <- aperm(coeff$ma, c(2, 1, 3))
-        }
-        R1 <- rbind(
-          R1,
-          t(
+        if (!transform_only) {
+          if (n_arima > 1) {
+            coeff$ar <- aperm(coeff$ar, c(2, 1, 3))
+          }
+          T1[1:(n_arima * arima_spec[[1]]), 1:n_arima] <- t(
             matrix(
-              coeff$ma,
+              coeff$ar,
               n_arima,
-              n_arima * arima_spec[3]
+              n_arima * arima_spec[[1]]
             )
           )
-        )
+        }
+      }
+      if (!transform_only) {
+        T_stationary <- T1
       }
-      R1 <- rbind(
-        R1,
-        matrix(0, (r - 1 - arima_spec[3]) * n_arima, n_arima)
-      )
-      R_stationary <- R1
 
-      # Non-stationary part
-      if (arima_spec[2] > 0) {
-        T1 <- cbind(T2, T1)
-        T1 <- rbind(T3, T1)
-        R1 <- rbind(R2, R1)
+      # R matrix coefficients
+      if (arima_spec[[3]] > 0) {
+        if (decompositions) {
+          result$ma <- coeff$ma
+        }
+        if (!transform_only) {
+          if (n_arima > 1) {
+            coeff$ma <- aperm(coeff$ma, c(2, 1, 3))
+          }
+          R1 <- rbind(
+            R1,
+            t(
+              matrix(
+                coeff$ma,
+                n_arima,
+                n_arima * arima_spec[[3]]
+              )
+            )
+          )
+        }
+      }
+      if (!transform_only) {
+        R1 <- rbind(
+          R1,
+          matrix(0, (r - 1 - arima_spec[[3]]) * n_arima, n_arima)
+        )
+        R_stationary <- R1
+
+        # Non-stationary part
+        if (arima_spec[[2]] > 0) {
+          T1 <- cbind(T2, T1)
+          T1 <- rbind(T3, T1)
+          R1 <- rbind(R2, R1)
+        }
+        result$Tmat <- T1
+        result$R <- R1
       }
-      result$Tmat <- T1
-      result$R <- R1
     }
 
-    # Initial uncertainty for the stationary part
-    if (arima_spec[1] == 0 & arima_spec[3] == 0) {
-      result$P_star <- BlockMatrix(
-        matrix(0, arima_spec[2] * n_arima, arima_spec[2] * n_arima),
-        result$Q
-      )
-    } else {
-      T_kronecker <- kronecker(T_stationary, T_stationary)
-      Tinv <- solve(diag(1, dim(T_kronecker)[1], dim(T_kronecker)[2]) - T_kronecker)
-      vecRQR <- matrix(R_stationary %*% result$Q %*% t(R_stationary))
-      vecPstar <- Tinv %*% vecRQR
-      result$P_star <- BlockMatrix(
-        matrix(0, arima_spec[2] * n_arima, arima_spec[2] * n_arima),
-        matrix(vecPstar, dim(T_stationary)[1], dim(T_stationary)[2])
-      )
+    if (!transform_only) {
+
+      # Initial uncertainty for the stationary part
+      if (arima_spec[[1]] == 0 && arima_spec[[3]] == 0) {
+        result$P_star <- BlockMatrix(
+          matrix(0, arima_spec[[2]] * n_arima, arima_spec[[2]] * n_arima),
+          result$Q
+        )
+      } else {
+        T_kronecker <- kronecker(T_stationary, T_stationary)
+        Tinv <- solve(diag(1, dim(T_kronecker)[[1]], dim(T_kronecker)[[2]]) - T_kronecker)
+        vecRQR <- matrix(tcrossprod(R_stationary %*% result$Q, R_stationary))
+        vecPstar <- Tinv %*% vecRQR
+        result$P_star <- BlockMatrix(
+          matrix(0, arima_spec[[2]] * n_arima, arima_spec[[2]] * n_arima),
+          matrix(vecPstar, dim(T_stationary)[[1]], dim(T_stationary)[[2]])
+        )
+      }
     }
   }
 
@@ -277,6 +304,7 @@ ARIMA <- function(p = 1,
 #' @inheritParams GetSysMat
 #' @inheritParams StateSpaceEval
 #' @inheritParams Cholesky
+#' @inheritParams Cycle
 #'
 #' @return
 #' A list containing the system matrices.
@@ -284,15 +312,16 @@ ARIMA <- function(p = 1,
 #' @noRd
 SARIMA <- function(p = 1,
                    sarima_spec = list(
-                     s  = c(1, 12),
+                     s = c(1, 12),
                      ar = c(1, 1),
-                     i  = c(0, 0),
+                     i = c(0, 0),
                      ma = c(0, 0)
                    ),
                    exclude_sarima = NULL,
                    fixed_part = TRUE,
                    update_part = TRUE,
-                   param = rep(1, 2 * p^2 + 0.5 * p * (p+1)),
+                   transform_only = FALSE,
+                   param = rep(1, 2 * p^2 + 0.5 * p * (p + 1)),
                    decompositions = TRUE,
                    T1 = NULL,
                    T2 = NULL,
@@ -302,19 +331,26 @@ SARIMA <- function(p = 1,
 
   # Check for erroneous input
   if (length(sarima_spec) != 4) {
-    stop("The SARIMA specification must be a list containing 4 vectors.")
+    stop(
+      "The SARIMA specification must be a list containing 4 vectors.",
+      call. = FALSE
+    )
   }
-  if (length(sarima_spec$s) != length(sarima_spec$ar) |
-      length(sarima_spec$s) != length(sarima_spec$i)  |
-      length(sarima_spec$s) != length(sarima_spec$ma)) {
-    stop("The vectors in the SARIMA specification must be of equal length.")
+  if (length(sarima_spec$s) != length(sarima_spec$ar) ||
+    length(sarima_spec$s) != length(sarima_spec$i) ||
+    length(sarima_spec$s) != length(sarima_spec$ma)) {
+    stop(
+      "The vectors in the SARIMA specification must be of equal length.",
+      call. = FALSE
+    )
   }
   if (min(sarima_spec$i) < 0) {
     stop(
       paste(
         "Number of differencing in the SARIMA specification must",
         "be greater than or equal to 0."
-      )
+      ),
+      call. = FALSE
     )
   }
 
@@ -353,7 +389,7 @@ SARIMA <- function(p = 1,
     )
 
     # T and R matrices are fixed if no coefficients are needed
-    if (sum(sarima_spec$ar) == 0 & sum(sarima_spec$ma) == 0) {
+    if (sum(sarima_spec$ar) == 0 && sum(sarima_spec$ma) == 0) {
       T1 <- matrix(0, n_sarima, n_sarima)
       if (sum(sarima_spec$i) > 0) {
         T1 <- cbind(
@@ -372,18 +408,20 @@ SARIMA <- function(p = 1,
             T2,
             do.call(cbind, T_list),
             cbind(
-              matrix(0,
-                     (s_aux[i] - 1) * n_sarima,
-                     sum(s_aux[aux < i]) * n_sarima
+              matrix(
+                0,
+                (s_aux[[i]] - 1) * n_sarima,
+                sum(s_aux[aux < i]) * n_sarima
               ),
-              diag((s_aux[i] - 1) * n_sarima),
-              matrix(0,
-                     (s_aux[i] - 1) * n_sarima,
-                     (1 + sum(s_aux[aux > i]) + r) * n_sarima
+              diag((s_aux[[i]] - 1) * n_sarima),
+              matrix(
+                0,
+                (s_aux[[i]] - 1) * n_sarima,
+                (1 + sum(s_aux[aux > i]) + r) * n_sarima
               )
             )
           )
-          T_list[[i]] <- matrix(0, dim(T_list[[i]])[1], dim(T_list[[i]])[2])
+          T_list[[i]] <- matrix(0, dim(T_list[[i]])[[1]], dim(T_list[[i]])[[2]])
         }
         T1 <- rbind(T2, T1)
       }
@@ -406,31 +444,36 @@ SARIMA <- function(p = 1,
 
       # Non-stationary part of Tmat and R
       if (sum(sarima_spec$i) > 0) {
-        T2 <- matrix(0, dim(T1)[1], sum(s_aux) * n_sarima)
-        T3 <- matrix(0, 0, dim(T1)[2] + dim(T2)[2])
+        T2 <- matrix(0, dim(T1)[[1]], sum(s_aux) * n_sarima)
+        T3 <- matrix(0, 0, dim(T1)[[2]] + dim(T2)[[2]])
         T_list <- lapply(
           s_aux,
           function(x) cbind(matrix(0, n_sarima, n_sarima * (x - 1)), diag(n_sarima))
         )
-        T_list <- c(T_list, list(diag(n_sarima), matrix(0, n_sarima, (r - 1) * n_sarima)))
+        T_list <- c(
+          T_list,
+          list(diag(n_sarima), matrix(0, n_sarima, (r - 1) * n_sarima))
+        )
         aux <- 1:length(s_aux)
         for (i in seq_along(s_aux)) {
           T3 <- rbind(
             T3,
             do.call(cbind, T_list),
             cbind(
-              matrix(0,
-                     (s_aux[i] - 1) * n_sarima,
-                     sum(s_aux[aux < i]) * n_sarima
+              matrix(
+                0,
+                (s_aux[[i]] - 1) * n_sarima,
+                sum(s_aux[aux < i]) * n_sarima
               ),
-              diag((s_aux[i] - 1) * n_sarima),
-              matrix(0,
-                     (s_aux[i] - 1) * n_sarima,
-                     (1 + sum(s_aux[aux > i]) + r) * n_sarima
+              diag((s_aux[[i]] - 1) * n_sarima),
+              matrix(
+                0,
+                (s_aux[[i]] - 1) * n_sarima,
+                (1 + sum(s_aux[aux > i]) + r) * n_sarima
               )
             )
           )
-          T_list[[i]] <- matrix(0, dim(T_list[[i]])[1], dim(T_list[[i]])[2])
+          T_list[[i]] <- matrix(0, dim(T_list[[i]])[[1]], dim(T_list[[i]])[[2]])
         }
         result$T2 <- T2
         result$T3 <- T3
@@ -447,10 +490,10 @@ SARIMA <- function(p = 1,
     needed <- 0.5 * n_sarima * (n_sarima + 1) +
       (sum(sarima_spec$ar) + sum(sarima_spec$ma)) * n_sarima^2
     if (length(param) < needed) {
-      stop("Not enough parameters supplied.")
+      stop("Not enough parameters supplied.", call. = FALSE)
     }
     if (length(param) > needed) {
-      stop("Too many parameters supplied.")
+      stop("Too many parameters supplied.", call. = FALSE)
     }
 
     # Parameters for the variance - covariance matrix
@@ -479,198 +522,214 @@ SARIMA <- function(p = 1,
     param <- param[-Q_indices]
 
     # T matrix and coefficients in R matrix
-    if (sum(sarima_spec$ar) > 0 | sum(sarima_spec$ma) > 0) {
+    if (sum(sarima_spec$ar) > 0 || sum(sarima_spec$ma) > 0) {
+      if (!transform_only) {
 
-      # Initialise coefficient vector/array for the AR part
-      if (n_sarima == 1) {
-        AR_coeff <- rep(0, sum(sarima_spec$s * sarima_spec$ar))
-      } else {
-        AR_coeff <- array(
-          0,
-          dim = c(n_sarima, n_sarima, sum(sarima_spec$s * sarima_spec$ar))
-        )
+        # Initialise coefficient vector/array for the AR part
+        if (n_sarima == 1) {
+          AR_coeff <- rep(0, sum(sarima_spec$s * sarima_spec$ar))
+        } else {
+          AR_coeff <- array(
+            0,
+            dim = c(n_sarima, n_sarima, sum(sarima_spec$s * sarima_spec$ar))
+          )
+        }
+        ar_polynomial <- c()
+
+        # Initialise coefficient vector/array for the MA part
+        if (n_sarima == 1) {
+          MA_coeff <- rep(0, sum(sarima_spec$s * sarima_spec$ma))
+        } else {
+          MA_coeff <- array(
+            0,
+            dim = c(n_sarima, n_sarima, sum(sarima_spec$s * sarima_spec$ma))
+          )
+        }
+        ma_polynomial <- c()
       }
-      ar_polynomial <- c()
-      if (fixed_part) {
+
+      # Initialise lists for AR and MA coefficients
+      if (decompositions) {
         sar <- list()
-      }
-
-      # Initialise coefficient vector/array for the MA part
-      if (n_sarima == 1) {
-        MA_coeff <- rep(0, sum(sarima_spec$s * sarima_spec$ma))
-      } else {
-        MA_coeff <- array(
-          0,
-          dim = c(n_sarima, n_sarima, sum(sarima_spec$s * sarima_spec$ma))
-        )
-      }
-      ma_polynomial <- c()
-      if (fixed_part) {
         sma <- list()
       }
 
       for (i in seq_along(sarima_spec$s)) {
-        if ((sarima_spec$ar[i] + sarima_spec$ma[i]) == 0) {
+        if ((sarima_spec$ar[[i]] + sarima_spec$ma[[i]]) == 0) {
           next
         }
 
         # Coefficients
-        ARMA_indices <- 1:((sarima_spec$ar[i] + sarima_spec$ma[i]) * n_sarima^2)
+        ARMA_indices <- 1:((sarima_spec$ar[[i]] + sarima_spec$ma[[i]]) * n_sarima^2)
         if (n_sarima == 1) {
           A <- param[ARMA_indices]
         } else {
           A <- array(
             param[ARMA_indices],
-            dim = c(n_sarima, n_sarima, sarima_spec$ar[i] + sarima_spec$ma[i])
+            dim = c(n_sarima, n_sarima, sarima_spec$ar[[i]] + sarima_spec$ma[[i]])
           )
         }
         coeff <- CoeffARMA(
           A = A, variance = result$Q,
-          ar = sarima_spec$ar[i], ma = sarima_spec$ma[i]
+          ar = sarima_spec$ar[[i]], ma = sarima_spec$ma[[i]]
         )
 
         # Update AR coefficient array
-        if (sarima_spec$ar[i] > 0) {
-          if (fixed_part) {
-            sar[[paste0("S", sarima_spec$s[i])]] <- coeff$ar
+        if (sarima_spec$ar[[i]] > 0) {
+          if (decompositions) {
+            sar[[paste0("S", sarima_spec$s[[i]])]] <- coeff$ar
           }
-          AR_coeff_old <- AR_coeff
-          add_coeff <- sarima_spec$s[i] * 1:sarima_spec$ar[i]
-          if (n_sarima == 1) {
-            AR_coeff[add_coeff] <- AR_coeff[add_coeff] + -coeff$ar
-            for (j in (1:sarima_spec$ar[i])) {
-              mult_coeff <- sarima_spec$s[i] * j + ar_polynomial
-              AR_coeff[mult_coeff] <- AR_coeff[mult_coeff] +
-                -coeff$ar[j] * AR_coeff_old[ar_polynomial]
-              add_coeff <- c(add_coeff, mult_coeff)
-            }
-          } else {
-            AR_coeff[,,add_coeff] <- AR_coeff[,,add_coeff, drop = FALSE] + -coeff$ar
-            for (j in (1:sarima_spec$ar[i])) {
-              mult_coeff <- sarima_spec$s[i] * j + ar_polynomial
-              AR_coeff[,,mult_coeff] <- AR_coeff[,,mult_coeff, drop = FALSE] +
-                array(
-                  -coeff$ar[,,j] %*%
-                    matrix(AR_coeff_old[,,ar_polynomial], nrow = n_sarima),
-                  dim = c(n_sarima, n_sarima, length(ar_polynomial))
-                )
-              add_coeff <- c(add_coeff, mult_coeff)
+          if (!transform_only) {
+            AR_coeff_old <- AR_coeff
+            add_coeff <- sarima_spec$s[[i]] * 1:sarima_spec$ar[[i]]
+            if (n_sarima == 1) {
+              AR_coeff[add_coeff] <- AR_coeff[add_coeff] - coeff$ar
+              for (j in (1:sarima_spec$ar[[i]])) {
+                mult_coeff <- sarima_spec$s[[i]] * j + ar_polynomial
+                AR_coeff[mult_coeff] <- AR_coeff[mult_coeff] -
+                  coeff$ar[[j]] * AR_coeff_old[ar_polynomial]
+                add_coeff <- c(add_coeff, mult_coeff)
+              }
+            } else {
+              AR_coeff[, , add_coeff] <- AR_coeff[, , add_coeff, drop = FALSE] - coeff$ar
+              for (j in (1:sarima_spec$ar[[i]])) {
+                mult_coeff <- sarima_spec$s[[i]] * j + ar_polynomial
+                AR_coeff[, , mult_coeff] <- AR_coeff[, , mult_coeff, drop = FALSE] -
+                  array(
+                    coeff$ar[, , j] %*%
+                      matrix(AR_coeff_old[, , ar_polynomial], nrow = n_sarima),
+                    dim = c(n_sarima, n_sarima, length(ar_polynomial))
+                  )
+                add_coeff <- c(add_coeff, mult_coeff)
+              }
             }
+            ar_polynomial <- unique(c(ar_polynomial, add_coeff))
           }
-          ar_polynomial <- unique(c(ar_polynomial, add_coeff))
         }
 
         # Update MA coefficient array
-        if (sarima_spec$ma[i] > 0) {
-          if (fixed_part) {
-            sma[[paste0("S", sarima_spec$s[i])]] <- coeff$ma
+        if (sarima_spec$ma[[i]] > 0) {
+          if (decompositions) {
+            sma[[paste0("S", sarima_spec$s[[i]])]] <- coeff$ma
           }
-          MA_coeff_old <- MA_coeff
-          add_coeff <- sarima_spec$s[i] * 1:sarima_spec$ma[i]
-          if (n_sarima == 1) {
-            MA_coeff[add_coeff] <- MA_coeff[add_coeff] + coeff$ma
-            for (j in (1:sarima_spec$ma[i])) {
-              mult_coeff <- sarima_spec$s[i] * j + ma_polynomial
-              MA_coeff[mult_coeff] <- MA_coeff[mult_coeff] +
-                coeff$ma[j] * MA_coeff_old[ma_polynomial]
-              add_coeff <- c(add_coeff, mult_coeff)
-            }
-          } else {
-            MA_coeff[,,add_coeff] <- MA_coeff[,,add_coeff, drop = FALSE] + coeff$ma
-            for (j in (1:sarima_spec$ma[i])) {
-              mult_coeff <- sarima_spec$s[i] * j + ma_polynomial
-              MA_coeff[,,mult_coeff] <- MA_coeff[,,mult_coeff, drop = FALSE] +
-                array(
-                  coeff$ma[,,j] %*%
-                    matrix(MA_coeff_old[,,ma_polynomial], nrow = n_sarima),
-                  dim = c(n_sarima, n_sarima, length(ma_polynomial))
-                )
-              add_coeff <- c(add_coeff, mult_coeff)
+          if (!transform_only) {
+            MA_coeff_old <- MA_coeff
+            add_coeff <- sarima_spec$s[[i]] * 1:sarima_spec$ma[[i]]
+            if (n_sarima == 1) {
+              MA_coeff[add_coeff] <- MA_coeff[add_coeff] + coeff$ma
+              for (j in (1:sarima_spec$ma[[i]])) {
+                mult_coeff <- sarima_spec$s[[i]] * j + ma_polynomial
+                MA_coeff[mult_coeff] <- MA_coeff[mult_coeff] +
+                  coeff$ma[[j]] * MA_coeff_old[ma_polynomial]
+                add_coeff <- c(add_coeff, mult_coeff)
+              }
+            } else {
+              MA_coeff[, , add_coeff] <- MA_coeff[, , add_coeff, drop = FALSE] + coeff$ma
+              for (j in (1:sarima_spec$ma[[i]])) {
+                mult_coeff <- sarima_spec$s[[i]] * j + ma_polynomial
+                MA_coeff[, , mult_coeff] <- MA_coeff[, , mult_coeff, drop = FALSE] +
+                  array(
+                    coeff$ma[, , j] %*%
+                      matrix(MA_coeff_old[, , ma_polynomial], nrow = n_sarima),
+                    dim = c(n_sarima, n_sarima, length(ma_polynomial))
+                  )
+                add_coeff <- c(add_coeff, mult_coeff)
+              }
             }
+            ma_polynomial <- unique(c(ma_polynomial, add_coeff))
           }
-          ma_polynomial <- unique(c(ma_polynomial, add_coeff))
         }
 
         # Drop parameters that are already used
         param <- param[-ARMA_indices]
       }
 
-      # T matrix coefficients
-      if (sum(sarima_spec$ar) > 0) {
-        if (n_sarima > 1) {
-          AR_coeff <- aperm(AR_coeff, c(2, 1, 3))
-        }
-        T1[1:(n_sarima * sum(sarima_spec$s * sarima_spec$ar)), 1:n_sarima] <- t(
-          matrix(
-            -AR_coeff,
-            n_sarima,
-            n_sarima * sum(sarima_spec$s * sarima_spec$ar)
-          )
-        )
-      }
-      T_stationary <- T1
+      if (!transform_only) {
 
-      # R matrix coefficients
-      if (sum(sarima_spec$ma) > 0) {
-        if (n_sarima > 1) {
-          MA_coeff <- aperm(MA_coeff, c(2, 1, 3))
+        # T matrix coefficients
+        if (sum(sarima_spec$ar) > 0) {
+          if (n_sarima > 1) {
+            AR_coeff <- aperm(AR_coeff, c(2, 1, 3))
+          }
+          T1[1:(n_sarima * sum(sarima_spec$s * sarima_spec$ar)), 1:n_sarima] <- t(
+            matrix(
+              -AR_coeff,
+              n_sarima,
+              n_sarima * sum(sarima_spec$s * sarima_spec$ar)
+            )
+          )
+        }
+        T_stationary <- T1
+
+        # R matrix coefficients
+        if (sum(sarima_spec$ma) > 0) {
+          if (n_sarima > 1) {
+            MA_coeff <- aperm(MA_coeff, c(2, 1, 3))
+          }
+          R1 <- rbind(
+            R1,
+            t(
+              matrix(
+                MA_coeff,
+                n_sarima,
+                n_sarima * sum(sarima_spec$s * sarima_spec$ma)
+              )
+            )
+          )
         }
         R1 <- rbind(
           R1,
-          t(
-            matrix(
-              MA_coeff,
-              n_sarima,
-              n_sarima * sum(sarima_spec$s * sarima_spec$ma)
-            )
+          matrix(
+            0,
+            (r - 1 - sum(sarima_spec$s * sarima_spec$ma)) * n_sarima,
+            n_sarima
           )
         )
-      }
-      R1 <- rbind(
-        R1,
-        matrix(
-          0,
-          (r - 1 - sum(sarima_spec$s * sarima_spec$ma)) * n_sarima,
-          n_sarima
-        )
-      )
-      R_stationary <- R1
+        R_stationary <- R1
 
-      # Non-stationary part
-      if (sum(sarima_spec$i) > 0) {
-        T1 <- cbind(T2, T1)
-        T1 <- rbind(T3, T1)
-        R1 <- rbind(R2, R1)
+        # Non-stationary part
+        if (sum(sarima_spec$i) > 0) {
+          T1 <- cbind(T2, T1)
+          T1 <- rbind(T3, T1)
+          R1 <- rbind(R2, R1)
+        }
+        result$Tmat <- T1
+        result$R <- R1
       }
-      result$Tmat <- T1
-      result$R <- R1
-      if (fixed_part) {
+
+      # Return AR and MA coefficients
+      if (decompositions) {
         result$sar <- sar
         result$sma <- sma
       }
     }
 
-    # Initial uncertainty for the stationary part
-    if (sum(sarima_spec$ar) == 0 & sum(sarima_spec$ma) == 0) {
-      result$P_star <- BlockMatrix(
-        matrix(0,
-               sum(sarima_spec$s * sarima_spec$i) * n_sarima,
-               sum(sarima_spec$s * sarima_spec$i) * n_sarima
-        ),
-        result$Q
-      )
-    } else {
-      T_kronecker <- kronecker(T_stationary, T_stationary)
-      Tinv <- solve(diag(1, dim(T_kronecker)[1], dim(T_kronecker)[2]) - T_kronecker)
-      vecRQR <- matrix(R_stationary %*% result$Q %*% t(R_stationary))
-      vecPstar <- Tinv %*% vecRQR
-      result$P_star <- BlockMatrix(
-        matrix(0,
-               sum(sarima_spec$s * sarima_spec$i) * n_sarima,
-               sum(sarima_spec$s * sarima_spec$i) * n_sarima
-        ),
-        matrix(vecPstar, dim(T_stationary)[1], dim(T_stationary)[2])
-      )
+    if (!transform_only) {
+
+      # Initial uncertainty for the stationary part
+      if (sum(sarima_spec$ar) == 0 && sum(sarima_spec$ma) == 0) {
+        result$P_star <- BlockMatrix(
+          matrix(
+            0,
+            sum(sarima_spec$s * sarima_spec$i) * n_sarima,
+            sum(sarima_spec$s * sarima_spec$i) * n_sarima
+          ),
+          result$Q
+        )
+      } else {
+        T_kronecker <- kronecker(T_stationary, T_stationary)
+        Tinv <- solve(diag(1, dim(T_kronecker)[[1]], dim(T_kronecker)[[2]]) - T_kronecker)
+        vecRQR <- matrix(tcrossprod(R_stationary %*% result$Q, R_stationary))
+        vecPstar <- Tinv %*% vecRQR
+        result$P_star <- BlockMatrix(
+          matrix(
+            0,
+            sum(sarima_spec$s * sarima_spec$i) * n_sarima,
+            sum(sarima_spec$s * sarima_spec$i) * n_sarima
+          ),
+          matrix(vecPstar, dim(T_stationary)[[1]], dim(T_stationary)[[2]])
+        )
+      }
     }
   }
 
