@@ -3,7 +3,7 @@
 #' Constructs the system matrices of a explanatory variables component.
 #'
 #' @inheritParams GetSysMat
-#' @inheritParams StateSpaceFit
+#' @inheritParams statespacer
 #' @inheritParams StateSpaceEval
 #' @inheritParams LocalLevel
 #' @inheritParams Cholesky
@@ -26,7 +26,8 @@ AddVar <- function(p = 1,
       paste(
         "The number of elements in `addvar_list` must",
         "be equal to the number of dependent variables."
-      )
+      ),
+      call. = FALSE
     )
   }
 
@@ -34,7 +35,13 @@ AddVar <- function(p = 1,
   diag_addvar <- sum(diag(format_addvar) != 0)
 
   # Number of coefficients
-  k <- sum(sapply(addvar_list, function(X) {if (is.null(X)) {0} else {dim(X)[2]}}))
+  k <- sum(
+    vapply(
+      addvar_list,
+      function(X) if (is.null(X)) 0L else dim(X)[[2]],
+      integer(1)
+    )
+  )
 
   # Initialising the list to return
   result <- list()
@@ -42,27 +49,28 @@ AddVar <- function(p = 1,
   if (fixed_part) {
 
     # Number of observations
-    N <- max(sapply(addvar_list, function(X) {if (is.null(X)) {0} else {dim(X)[1]}}))
+    N <- max(
+      vapply(
+        addvar_list,
+        function(X) if (is.null(X)) 0L else dim(X)[[1]],
+        integer(1)
+      )
+    )
 
     # Z = a p x k x N array
     Ztemp <- do.call(BlockMatrix, addvar_list)
     index <- 1:N
-    Ztemp2 <- NULL
-    null_mat <- matrix(0, N, k)
+    Ztemp2 <- matrix(0, p * N, k)
     for (i in seq_along(addvar_list)) {
-      if (is.null(addvar_list[[i]])) {
-        Ztemp2 <- rbind(Ztemp2, null_mat)
-      } else {
-        Ztemp2 <- rbind(Ztemp2, Ztemp[index,])
+      if (!is.null(addvar_list[[i]])) {
+        Ztemp2[1:N + (i - 1) * N, ] <- Ztemp[index, ]
         index <- index + N
       }
     }
-    result$Z <- array(
-      sapply(
-        seq(N),
-        function(i) {Ztemp2[seq(i, p * N, N),, drop = FALSE]},
-        simplify = "array"
-      ), dim = c(p, k, N)
+    result$Z <- vapply(
+      seq(N),
+      function(i) Ztemp2[seq(i, p * N, N), , drop = FALSE],
+      matrix(0, p, k)
     )
 
     # T matrix = a k x k diagonal matrix, containing ones on the diagonal
@@ -82,15 +90,16 @@ AddVar <- function(p = 1,
     }
   }
 
-  if (update_part & diag_addvar > 0) {
+  if (update_part && diag_addvar > 0) {
 
     # Check whether the number of rows of format_addvar is valid
-    if (dim(format_addvar)[1] != k) {
+    if (dim(format_addvar)[[1]] != k) {
       stop(
         paste(
           "The number of rows of `format_addvar` for the explanatory variables",
           "must be equal to the number of explanatory variables."
-        )
+        ),
+        call. = FALSE
       )
     }
 
@@ -124,7 +133,7 @@ AddVar <- function(p = 1,
 #' in the level component.
 #'
 #' @inheritParams GetSysMat
-#' @inheritParams StateSpaceFit
+#' @inheritParams statespacer
 #' @inheritParams StateSpaceEval
 #' @inheritParams LocalLevel
 #' @inheritParams Cholesky
@@ -149,7 +158,8 @@ LevelAddVar <- function(p = 1,
       paste(
         "The number of elements in `level_addvar_list`",
         "must be equal to the number of dependent variables."
-      )
+      ),
+      call. = FALSE
     )
   }
 
@@ -162,16 +172,28 @@ LevelAddVar <- function(p = 1,
   # Variable that is used to check if Q_level_addvar should be a matrix of 0s
   diag_level_addvar <- sum(diag(format_level_addvar) != 0)
 
+  # Number of coefficients
+  k <- sum(
+    vapply(
+      level_addvar_list,
+      function(X) if (is.null(X)) 0L else dim(X)[[2]],
+      integer(1)
+    )
+  )
+
   # Initialising the list to return
   result <- list()
 
   if (fixed_part) {
 
-    # Number of coefficients
-    k <- sum(sapply(level_addvar_list, function(X) {if (is.null(X)) {0} else {dim(X)[2]}}))
-
     # Number of observations
-    N <- max(sapply(level_addvar_list, function(X) {if (is.null(X)) {0} else {dim(X)[1]}}))
+    N <- max(
+      vapply(
+        level_addvar_list,
+        function(X) if (is.null(X)) 0L else dim(X)[[1]],
+        integer(1)
+      )
+    )
 
     # Z matrix = matrix of p rows and n_level + k columns
     #   First n_level columns containing one 1, 0s elsewhere
@@ -185,41 +207,41 @@ LevelAddVar <- function(p = 1,
     # Tmat = a (n_level + k) x (n_level + k) x N array
     Ttemp <- do.call(BlockMatrix, level_addvar_list)
     index <- 1:N
-    Ttemp2 <- NULL
-    null_mat <- matrix(0, N, k)
+    Ttemp2 <- matrix(0, n_level * N, k)
+    j <- 1
     for (i in seq_along(level_addvar_list)) {
-      if (is.null(level_addvar_list[[i]])) {
-        if (!(i %in% exclude_level)) {
-          Ttemp2 <- rbind(Ttemp2, null_mat)
-        }
-      } else {
+      if (!is.null(level_addvar_list[[i]])) {
         if (i %in% exclude_level) {
           stop(
             paste0(
               "The ", i, "th dependent variable was specified to be excluded ",
               "from getting a local level, ",
               "but did have explanatory variables specified for the local level."
-            )
+            ),
+            call. = FALSE
           )
         } else {
-          Ttemp2 <- rbind(Ttemp2, Ttemp[index,])
+          Ttemp2[1:N + (j - 1) * N, ] <- Ttemp[index, ]
           index <- index + N
+          j <- j + 1
         }
       }
     }
-    result$Tmat <- sapply(
+    result$Tmat <- vapply(
       seq(N),
-      function(i) {rbind(
-        cbind(
-          diag(1, n_level, n_level),
-          Ttemp2[seq(i, n_level * N, N),, drop = FALSE]
-        ),
-        cbind(
-          matrix(0, k, n_level),
-          diag(1, k, k)
+      function(i) {
+        rbind(
+          cbind(
+            diag(1, n_level, n_level),
+            Ttemp2[seq(i, n_level * N, N), , drop = FALSE]
+          ),
+          cbind(
+            matrix(0, k, n_level),
+            diag(1, k, k)
+          )
         )
-      )},
-      simplify = "array"
+      },
+      matrix(0, k + n_level, k + n_level)
     )
 
     # R matrix = a (n_level + k) x (n_level + k) diagonal matrix containing 1s
@@ -241,19 +263,19 @@ LevelAddVar <- function(p = 1,
     }
   }
 
-  if (update_part & (diag_level + diag_level_addvar) > 0) {
-
+  if (update_part && (diag_level + diag_level_addvar) > 0) {
     if (diag_level > 0) {
 
       # Check whether the number of rows of format_level is valid
-      if (dim(format_level)[1] != n_level) {
+      if (dim(format_level)[[1]] != n_level) {
         stop(
           paste(
             "The number of rows of `format_level` for the level +",
             "explanatory variables in the level component must be equal",
             "to the number of dependent variables minus the number of",
             "excluded dependent variables."
-          )
+          ),
+          call. = FALSE
         )
       }
 
@@ -278,7 +300,6 @@ LevelAddVar <- function(p = 1,
       } else {
         result$Q_level <- Q_level
       }
-
     } else {
       split <- 0
     }
@@ -286,20 +307,21 @@ LevelAddVar <- function(p = 1,
     if (diag_level_addvar > 0) {
 
       # Check whether the number of rows of format_level_addvar is valid
-      if (dim(format_level_addvar)[1] != k) {
+      if (dim(format_level_addvar)[[1]] != k) {
         stop(
           paste(
             "The number of rows of `format_level_addvar` for the level +",
             "explanatory variables in the level component",
             "must be equal to the number of explanatory variables."
-          )
+          ),
+          call. = FALSE
         )
       }
 
       # Using Cholesky function to get a valid variance - covariance matrix
       # for the Q matrix for the level_addvar
       Q_level_addvar <- Cholesky(
-        param = param[(split + 1) : length(param)],
+        param = param[(split + 1):length(param)],
         format = format_level_addvar,
         decompositions = decompositions
       )
@@ -327,7 +349,7 @@ LevelAddVar <- function(p = 1,
 #' level + slope component.
 #'
 #' @inheritParams GetSysMat
-#' @inheritParams StateSpaceFit
+#' @inheritParams statespacer
 #' @inheritParams StateSpaceEval
 #' @inheritParams LocalLevel
 #' @inheritParams Cholesky
@@ -354,7 +376,8 @@ SlopeAddVar <- function(p = 1,
       paste(
         "The number of elements in `level_addvar_list` must be equal",
         "to the number of dependent variables."
-      )
+      ),
+      call. = FALSE
     )
   }
 
@@ -373,24 +396,26 @@ SlopeAddVar <- function(p = 1,
   # Variable that is used to check if Q_level_addvar should be a matrix of 0s
   diag_level_addvar <- sum(diag(format_level_addvar) != 0)
 
+  # Number of coefficients
+  k <- sum(
+    vapply(
+      level_addvar_list,
+      function(X) if (is.null(X)) 0L else dim(X)[[2]],
+      integer(1)
+    )
+  )
+
   # Initialising the list to return
   result <- list()
 
   if (fixed_part) {
 
-    # Number of coefficients
-    k <- sum(
-      sapply(
-        level_addvar_list,
-        function(X) {if (is.null(X)) {0} else {dim(X)[2]}}
-      )
-    )
-
     # Number of observations
     N <- max(
-      sapply(
+      vapply(
         level_addvar_list,
-        function(X) {if (is.null(X)) {0} else {dim(X)[1]}}
+        function(X) if (is.null(X)) 0L else dim(X)[[1]],
+        integer(1)
       )
     )
 
@@ -407,25 +432,23 @@ SlopeAddVar <- function(p = 1,
     # Tmat = a (n_level + n_slope + k) x (n_level + n_slope + k) x N array
     Ttemp <- do.call(BlockMatrix, level_addvar_list)
     index <- 1:N
-    Ttemp2 <- NULL
-    null_mat <- matrix(0, N, k)
+    Ttemp2 <- matrix(0, n_level * N, k)
+    j <- 1
     for (i in seq_along(level_addvar_list)) {
-      if (is.null(level_addvar_list[[i]])) {
-        if (!(i %in% exclude_level)) {
-          Ttemp2 <- rbind(Ttemp2, null_mat)
-        }
-      } else {
+      if (!is.null(level_addvar_list[[i]])) {
         if (i %in% exclude_level) {
           stop(
             paste0(
               "The ", i, "th dependent variable was specified to be excluded ",
               "from getting a local level, ",
               "but did have explanatory variables specified for the local level."
-            )
+            ),
+            call. = FALSE
           )
         } else {
-          Ttemp2 <- rbind(Ttemp2, Ttemp[index,])
+          Ttemp2[1:N + (j - 1) * N, ] <- Ttemp[index, ]
           index <- index + N
+          j <- j + 1
         }
       }
     }
@@ -443,22 +466,24 @@ SlopeAddVar <- function(p = 1,
       exclude <- c(exclude_level, p + exclude_slope)
       Ttemp3 <- Ttemp3[-exclude, -exclude, drop = FALSE]
     }
-    result$Tmat <- sapply(
+    result$Tmat <- vapply(
       seq(N),
-      function(i) {rbind(
-        cbind(
-          Ttemp3,
-          rbind(
-            Ttemp2[seq(i, n_level * N, N),, drop = FALSE],
-            matrix(0, n_slope, k)
+      function(i) {
+        rbind(
+          cbind(
+            Ttemp3,
+            rbind(
+              Ttemp2[seq(i, n_level * N, N), , drop = FALSE],
+              matrix(0, n_slope, k)
+            )
+          ),
+          cbind(
+            matrix(0, k, n_level + n_slope),
+            diag(1, k, k)
           )
-        ),
-        cbind(
-          matrix(0, k, n_level + n_slope),
-          diag(1, k, k)
         )
-      )},
-      simplify = "array"
+      },
+      matrix(0, k + n_level + n_slope, k + n_level + n_slope)
     )
 
     # R matrix = a (n_level + n_slope + k) x (n_level + n_slope + k)
@@ -486,19 +511,19 @@ SlopeAddVar <- function(p = 1,
     }
   }
 
-  if (update_part & (diag_level + diag_slope + diag_level_addvar) > 0) {
-
+  if (update_part && (diag_level + diag_slope + diag_level_addvar) > 0) {
     if (diag_level > 0) {
 
       # Check whether the number of rows of format_level is valid
-      if (dim(format_level)[1] != n_level) {
+      if (dim(format_level)[[1]] != n_level) {
         stop(
           paste(
             "The number of rows of `format_level` for the level",
             "+ slope + explanatory variables in the level component must be",
             "equal to the number of dependent variables minus",
             "the number of excluded dependent variables."
-          )
+          ),
+          call. = FALSE
         )
       }
 
@@ -523,7 +548,6 @@ SlopeAddVar <- function(p = 1,
       } else {
         result$Q_level <- Q_level
       }
-
     } else {
       split <- 0
     }
@@ -531,14 +555,15 @@ SlopeAddVar <- function(p = 1,
     if (diag_slope > 0) {
 
       # Check whether the number of rows of format_slope is valid
-      if (dim(format_slope)[1] != n_slope) {
+      if (dim(format_slope)[[1]] != n_slope) {
         stop(
           paste(
             "The number of rows of `format_slope` for the level",
             "+ slope + explanatory variables in the level component must be",
             "equal to the number of local levels minus",
             "the number of excluded local levels."
-          )
+          ),
+          call. = FALSE
         )
       }
 
@@ -563,7 +588,6 @@ SlopeAddVar <- function(p = 1,
       } else {
         result$Q_slope <- Q_slope
       }
-
     } else {
       split2 <- 0
     }
@@ -571,20 +595,21 @@ SlopeAddVar <- function(p = 1,
     if (diag_level_addvar > 0) {
 
       # Check whether the number of rows of format_level_addvar is valid
-      if (dim(format_level_addvar)[1] != k) {
+      if (dim(format_level_addvar)[[1]] != k) {
         stop(
           paste(
             "The number of rows of `format_level_addvar` for the level",
             "+ slope + explanatory variables in the level component must be",
             "equal to the number of explanatory variables."
-          )
+          ),
+          call. = FALSE
         )
       }
 
       # Using Cholesky function to get a valid variance - covariance matrix
       # for the Q matrix for the level_addvar
       Q_level_addvar <- Cholesky(
-        param = param[(split + split2 + 1) : length(param)],
+        param = param[(split + split2 + 1):length(param)],
         format = format_level_addvar,
         decompositions = decompositions
       )
